<?php

namespace Drupal\media_twig_tools\Config;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\File\FileSystemInterface;

/**
 * Configuration for MediaTwigTools settings.
 *
 * @package Drupal\media_twig_tools\Config
 */
class MediaTwigToolsConfig implements MediaTwigToolsConfigInterface {

  /**
   * Nom du service.
   *
   * @const string
   */
  const SERVICE_NAME = 'media_twig_tools.config';

  /**
   * Field dirname.
   *
   * @const string
   */
  const FIELD_DIRNAME = 'dirname';

  /**
   * Field no style.
   *
   * @const string
   */
  const FIELD_NO_STYLE = 'no_style';

  /**
   * Default no style identifier.
   *
   * @const string
   */
  const DEFAULT_NO_STYLE_IDENTIFIER = 'unstyle';

  /**
   * Config.
   *
   * @var \Drupal\Core\Config\Config
   */
  protected $conf;

  /**
   * The file System.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * File storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected EntityStorageInterface $fileStorage;

  /**
   * Image style storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected EntityStorageInterface $imageStyleStorage;

  /**
   * MediaTwigToolsConfig constructor.
   */
  public function __construct(ConfigFactoryInterface $configFactory, FileSystemInterface $fileSystem, EntityTypeManagerInterface $entityTypeManager) {
    $this->conf = $configFactory->getEditable(static::SERVICE_NAME);
    $this->fileSystem = $fileSystem;
    $this->fileStorage = $entityTypeManager->getStorage('file');
    $this->imageStyleStorage = $entityTypeManager->getStorage('image_style');
  }

  /**
   * Retourne le singleton.
   *
   * @return static
   *   Le singleton.
   */
  public static function me() {
    return \Drupal::service(static::SERVICE_NAME);
  }

  /**
   * Return true if $value is a valid no style identifier.
   *
   * @param string $value
   *   The value.
   *
   * @return bool
   *   The result.
   */
  public function existsNoStyleIdentifier($value) {
    return array_key_exists($value, $this->getImageStyleStorage()->loadMultiple());
  }

  /**
   * Return the dirname.
   *
   * @return string
   *   The dirname.
   */
  public function getDirname() {
    return $this->conf->get(static::FIELD_DIRNAME) ?: 'builder';
  }

  /**
   * Update the dirname.
   *
   * @param string $dirname
   *   The dirname.
   *
   * @return MediaTwigToolsConfig
   *   The config.
   */
  public function setDirname($dirname) {
    $this->conf
      ->set(static::FIELD_DIRNAME, $dirname)
      ->save();

    return $this;
  }

  /**
   * Return the no style identifier.
   *
   * @return string
   *   The no style identifier.
   */
  public function getNoStyle() {
    return $this->conf->get(static::FIELD_NO_STYLE) ?: static::DEFAULT_NO_STYLE_IDENTIFIER;
  }

  /**
   * Update the no style identifier.
   *
   * @param string $noStyle
   *   The no style identifier.
   *
   * @return MediaTwigToolsConfig
   *   The config.
   */
  public function setNoStyle($noStyle) {
    $this->conf
      ->set(static::FIELD_NO_STYLE, $noStyle)
      ->save();

    return $this;
  }

  /**
   * Return the list of dirnames.
   *
   * @param bool $force
   *   Force retreival without cache.
   *
   * @return array[]
   *   The list of dirs by type.
   */
  public function getDirs($force = FALSE) {
    if ($force || empty($this->dirs)) {
      $this->dirs = ['public' => [], 'private' => []];
      foreach ($this->dirs as $key => &$dir) {
        $dir['path'] = $this->fileSystem->realpath($key . '://' . $this->getDirname());
        try {
          $dir['files'] = glob($dir['path'] . '/*');
        }
        catch (\Exception $e) {
          $dir['files'] = [];
        }
      }
    }

    return $this->dirs;
  }

  /**
   * Clear generated images dir.
   *
   * @param string $dirname
   *   The dirname.
   */
  public function clearDir($dirname) {
    $this->fileSystem->deleteRecursive($dirname);
  }

  /**
   * REturn the file storage.
   *
   * @return \Drupal\Core\Entity\EntityStorageInterface
   *   The file storage.
   */
  public function getFileStorage(): EntityStorageInterface {
    return $this->fileStorage;
  }

  /**
   * Return the image style storage.
   *
   * @return \Drupal\Core\Entity\EntityStorageInterface
   *   The image style storage.
   */
  public function getImageStyleStorage(): EntityStorageInterface {
    return $this->imageStyleStorage;
  }

}
