<?php

namespace Drupal\media_twig_tools\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\media_twig_tools\Config\MediaTwigToolsConfig;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a Media Twig Tools form.
 */
class SettingsForm extends FormBase {

  /**
   * Conf.
   *
   * @var \Drupal\media_twig_tools\Config\MediaTwigToolsConfig
   */
  protected $conf;

  /**
   * Dirs list.
   *
   * @var array
   */
  protected $dirs;

  /**
   * SettingsForm constructor.
   *
   * @param \Drupal\media_twig_tools\Config\MediaTwigToolsConfig $conf
   *   The media twig tools config.
   */
  public function __construct(MediaTwigToolsConfig $conf) {
    $this->conf = $conf;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get(MediaTwigToolsConfig::SERVICE_NAME),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'media_twig_tools_settings';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {

    // Clear.
    $form['clearer'] = $this->getClearerFormPart($form_state);

    // Settings.
    $form['settings'] = $this->getSettingsFormPart($form_state, empty($form['clearer']));

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $triggeringElement = $form_state->getTriggeringElement();
    if (isset($triggeringElement['#submit_action']) && $submitAction = $triggeringElement['#submit_action']) {
      switch ($submitAction) {
        case 'clear':
          $this->clearImages($form_state);
          break;

        case 'save':
          $this->saveSettings($form_state);
          break;

      }
    }
  }

  /**
   * Init settings.
   *
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The formstate.
   * @param bool $open
   *   The part should be open by default.
   *
   * @return array
   *   The settings form part.
   */
  protected function getSettingsFormPart(FormStateInterface $form_state, $open = FALSE) {
    $settings = [
      '#type'  => 'details',
      '#title' => $this->t('Settings'),
      '#tree'  => TRUE,
      '#open'  => $open,
    ];

    $settings[MediaTwigToolsConfig::FIELD_DIRNAME] = [
      '#type'          => 'textfield',
      '#title'         => $this->t('Built image directory name'),
      '#default_value' => $this->conf->getDirname(),
      '#description'   => $this->t('The name of the directory of the generated images'),
      '#required'      => TRUE,
    ];

    $settings[MediaTwigToolsConfig::FIELD_NO_STYLE] = [
      '#type'          => 'machine_name',
      '#machine_name'  => [
        'exists'          => [$this->conf, 'existsNoStyleIdentifier'],
        'replace_pattern' => '[^a-z0-9_]+',
      ],
      '#title'         => $this->t('No image style identifier'),
      '#description'   => $this->t('The rule identifier of a no styled-image base'),
      '#default_value' => $this->conf->getNoStyle(),
      '#required'      => TRUE,
    ];

    $settings['actions'] = [
      '#type'  => 'actions',
      'submit' => [
        '#type'          => 'submit',
        '#value'         => $this->t('Save settings'),
        '#submit_action' => 'save',
      ],
    ];

    return $settings;
  }

  /**
   * Init the clear images form part.
   *
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   *
   * @return array
   *   The form part.
   */
  protected function getClearerFormPart(FormStateInterface $form_state) {
    $clearer = [
      '#type'   => 'details',
      '#title'  => $this->t('Clear generated images'),
      '#open'   => TRUE,
      '#tree'   => TRUE,
      'actions' => [
        '#type'  => 'actions',
        'submit' => [
          '#type'          => 'submit',
          '#value'         => $this->t('Delete generated images'),
          '#submit_action' => 'clear',
        ],
      ],
    ];

    // List of builder dirs.
    foreach ($this->conf->getDirs() as $type => $data) {
      $options = array_combine($data['files'], $data['files']);
      $options = array_map(function ($path) {
        $basename = basename($path);
        if ($imageStyle = $this->conf->getImageStyleStorage()->load($basename)) {
          return $imageStyle->label();
        }
        else {
          return $this->t('No image style') . '(' . $basename . ')';
        }
      }, $options);

      if (!empty($data['files'])) {
        $clearer['types'][$type] = [
          '#type'    => 'checkboxes',
          '#title'   => $this->t($type),
          '#options' => $options,
        ];
      }
    }

    if (!empty($clearer['types'])) {
      return $clearer;
    }

    return [];
  }

  /**
   * Save settings action.
   *
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   */
  protected function saveSettings(FormStateInterface $form_state) {
    $settings = $form_state->getValue('settings');
    $this->conf
      ->setDirname($settings[MediaTwigToolsConfig::FIELD_DIRNAME])
      ->setNoStyle($settings[MediaTwigToolsConfig::FIELD_NO_STYLE]);
  }

  /**
   * Clear images action.
   *
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The formstate.
   */
  protected function clearImages(FormStateInterface $form_state) {
    foreach ($form_state->getValues()['clearer']['types'] as $type) {
      foreach (array_filter($type) as $dirname) {
        $this->conf->clearDir($dirname);
      }
    }
  }

}
