<?php

namespace Drupal\media_twig_tools\Generators;

use Drupal\file\Entity\File;
use Drupal\image\Entity\ImageStyle;
use Drupal\media_twig_tools\Services\ImageBuilder;

/**
 * Generate images based on options.
 *
 * @package Drupal\media_twig_tools\Generators
 */
class ImageGenerator {

  /**
   * The ImageBuilder service.
   *
   * @var \Drupal\media_twig_tools\Services\ImageBuilder
   */
  protected $imageBuilder;

  /**
   * Data.
   *
   * @var string
   */
  protected $pathData;

  /**
   * File.
   *
   * @var string
   */
  protected $file;

  /**
   * Rules.
   *
   * @var string
   */
  protected $rules;

  /**
   * Style.
   *
   * @var string
   */
  protected $style;

  /**
   * Dirname.
   *
   * @var string
   */
  protected $dirname;

  /**
   * Filename.
   *
   * @var string
   */
  protected $filename;

  /**
   * Extension.
   *
   * @var string
   */
  protected $extension;

  /**
   * ImageGenerator constructor.
   */
  public function __construct($pathData, ImageBuilder $imageBuilder) {
    $this->imageBuilder = $imageBuilder;
    $this->pathData = $pathData;
    $this->file = File::load($pathData['f']);
    $this->rules = $pathData['rules'];
    $this->style = $pathData['style'] === $this->imageBuilder->getNoStyleName() ? NULL : ImageStyle::load($pathData['style']);
    $this->style = $this->style ?: ImageStyle::create([]);
    $this->dirname = $pathData['dirname'];
    $this->filename = $pathData['filename'];
    $this->extension = $pathData['extension'];
  }

  /**
   * Generate the image.
   *
   * @return bool
   *   True if file exists;
   */
  public function generate() {
    if ($this->fileExists()) {
      return TRUE;
    }

    // Init weight.
    $effects = $this->style->getEffects()->getConfiguration();
    $weight = count($effects) > 0 ? end($effects)['weight'] : 0;

    // Add image style rules.
    foreach ($this->rules as $rule) {
      $this->style->addImageEffect($rule + ['weight' => $weight++]);
    }

    // Generate the image.
    $this->style->createDerivative($this->file->getFileUri(), $this->getDestination());

    return $this->fileExists();
  }

  /**
   * Return the destination of the generated file.
   *
   * @return string
   *   The destination.
   */
  public function getDestination() {
    $path = ($this->dirname[0] === '/' ? substr($this->dirname, 1) : $this->dirname) . '/' . $this->filename . '.' . $this->extension;

    return urldecode($path);
  }

  /**
   * Check if generated file already exists.
   *
   * @return bool
   *   File exists.
   */
  public function fileExists() {
    $destination = $this->getDestination();

    return file_exists($destination);
  }

}
