<?php

namespace Drupal\media_twig_tools\Services;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Field\EntityReferenceFieldItemList;
use Drupal\Core\Field\Plugin\Field\FieldType\EntityReferenceItem;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Render\Element;
use Drupal\file\FileInterface;
use Drupal\file\Plugin\Field\FieldType\FileFieldItemList;
use Drupal\image\ImageStyleInterface;
use Drupal\media\Entity\Media;
use Drupal\media\MediaInterface;
use Drupal\media_twig_tools\Config\MediaTwigToolsConfig;
use Twig\Extension\AbstractExtension;
use Twig\TwigFunction;

/**
 * Provides twig tools for facilitate media integration.
 *
 * - Render the image tag directly :
 *    {{ imgFromMedia(content.field_media_1, {style:'my_style', class:'class1
 * class2', title:'My title', {attributes: {'data-attr-1':'1']}}) }}
 *
 * @package Drupal\media_twig_tools\Services
 */
class MediaTwigTools extends AbstractExtension {

  /**
   * OPTION style.
   *
   * @const string
   */
  const OPTION_STYLE = 'style';

  /**
   * OPTION nom du champ.
   *
   * @const string
   */
  const OPTION_FIELD_NAME = 'field_name';

  /**
   * OPTION class.
   *
   * @const string
   */
  const OPTION_CLASS = 'class';

  /**
   * OPtion title.
   *
   * @const string
   */
  const OPTION_TITLE = 'title';

  /**
   * OPTION dimension.
   *
   * @const string
   */
  const OPTION_DIMENSION = 'dimension';

  /**
   * OPTION Settings.
   *
   * @const string
   */
  const OPTION_SETTINGS = 'settings';

  /**
   * OPTION relative url.
   *
   * @const strings
   */
  const OPTION_RELATIVE = 'relative';

  /**
   * OPTION attribute.
   *
   * @const string
   */
  const OPTION_ATTRIBUTES = 'attributes';

  /**
   * OPTION rules.
   *
   * @const string
   */
  const OPTION_RULES = 'rules';

  /**
   * OPTION no-size.
   *
   * @const string
   */
  const OPTION_NO_SIZE = 'no_size';

  /**
   * OPTION Override src.
   *
   * @const string
   */
  public const OPTION_OVERRIDE_SRC = 'data-override-src';

  /**
   * Nom du service.
   *
   * @const string
   */
  const SERVICE_NAME = 'media_twig_tools.media_twig_tools';

  /**
   * Entity TYpe manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * LanguageManager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * ImageBuilder.
   *
   * @var \Drupal\media_twig_tools\Services\ImageBuilder
   */
  protected $imageBuilder;

  /**
   * The configuration.
   *
   * @var \Drupal\media_twig_tools\Config\MediaTwigToolsConfig
   */
  protected MediaTwigToolsConfig $conf;

  /**
   * The file system.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected FileSystemInterface $fileSystem;

  /**
   * TwigUtils constructor.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   * @param \Drupal\Core\Language\LanguageManagerInterface $languageManager
   *   The language Manager.
   * @param \Drupal\media_twig_tools\Services\ImageBuilder $imageBuilder
   *   The image builder.
   * @param \Drupal\media_twig_tools\Config\MediaTwigToolsConfig $conf
   *   The configuration.
   * @param \Drupal\Core\File\FileSystemInterface $fileSystem
   *   THe file system.
   */
  public function __construct(EntityTypeManagerInterface $entityTypeManager, LanguageManagerInterface $languageManager, ImageBuilder $imageBuilder, MediaTwigToolsConfig $conf, FileSystemInterface $fileSystem) {
    $this->entityTypeManager = $entityTypeManager;
    $this->languageManager = $languageManager;
    $this->imageBuilder = $imageBuilder;
    $this->conf = $conf;
    $this->fileSystem = $fileSystem;
  }

  /**
   * Retourne le singleton.
   *
   * @return static
   *   Le singleton.
   */
  public static function me() {
    return \Drupal::service(static::SERVICE_NAME);
  }

  /**
   * {@inheritdoc}
   */
  public function getFunctions() {
    return [
      new TwigFunction('imgFromMedia', [$this, 'imgFromMedia']),
      new TwigFunction('getMediaUrl', [$this, 'getMediaUrl']),
      new TwigFunction('pictureFromMedia', [$this, 'pictureFromMedia']),
    ];
  }

  /**
   * Return the default options.
   *
   * @param array $options
   *   The user options.
   *
   * @return array
   *   THe merged options
   */
  public function getDefaultOptions(array $options) {
    $style = isset($options[static::OPTION_STYLE]) ? $options[static::OPTION_STYLE] : NULL;
    if ($style) {
      $style = $style instanceof ImageStyleInterface ? $style : $this->conf->getImageStyleStorage()
        ->load($style);
    }

    return [
      static::OPTION_STYLE      => $style,
      static::OPTION_CLASS      => isset($options[static::OPTION_CLASS]) ? $options[static::OPTION_CLASS] : NULL,
      static::OPTION_FIELD_NAME => isset($options[static::OPTION_FIELD_NAME]) ? $options[static::OPTION_FIELD_NAME] : 'field_media_image',
      static::OPTION_TITLE      => isset($options[static::OPTION_TITLE]) ? $options[static::OPTION_TITLE] : NULL,
      static::OPTION_DIMENSION  => isset($options[static::OPTION_DIMENSION]) ? TRUE : FALSE,
      static::OPTION_SETTINGS   => isset($options[static::OPTION_SETTINGS]) ? $options[static::OPTION_SETTINGS] : [],
      static::OPTION_RELATIVE   => isset($options[static::OPTION_RELATIVE]) ? $options[static::OPTION_RELATIVE] : TRUE,
      static::OPTION_ATTRIBUTES => isset($options[static::OPTION_ATTRIBUTES]) ? $options[static::OPTION_ATTRIBUTES] : [],
      static::OPTION_NO_SIZE    => isset($options[static::OPTION_NO_SIZE]) ? $options[static::OPTION_NO_SIZE] : TRUE,
    ] + $options;
  }

  /**
   * Retourne un build array d'image depuis un media.
   *
   * @param mixed $mediaParams
   *   Le media ou build array de media.
   * @param array $options
   *   Les options.
   *
   * @return array
   *   Le build array de l'image.
   */
  public function imgFromMedia($mediaParams, array $options = []) {
    // Définitions des options par défaut.
    $options = $this->getDefaultOptions($options);

    // Pour chaque média, on retourne le build array.
    $buildArray = [];
    foreach ($this->getMediaEntityFromTwigParam($mediaParams) as $media) {
      if ($build = $this->getImageBuildArrayFromMedia($media, $options)) {
        $buildArray[] = $build;
      }
    }

    return $buildArray;
  }

  /**
   * Retourne la  liste des entités media depuis le paramètre twig passé.
   *
   * @param mixed $mediaParams
   *   The media params.
   *
   * @return array
   *   The media entity list.
   */
  public function getMediaEntityFromTwigParam($mediaParams) {
    $mediaList = [];
    if (is_array($mediaParams)) {
      // Parcours du tableau de média.
      foreach (Element::children($mediaParams) as $key) {
        if ($mediaParams[$key] instanceof Media) {
          // Liste de média brut.
          $mediaList[] = $mediaParams[$key];
        }
        elseif ($mediaParams[$key]['#media'] instanceof Media) {
          // List de build array.
          $mediaList[] = $mediaParams[$key]['#media'];
        }
      }
    }
    elseif ($mediaParams instanceof Media) {
      $mediaList = [$mediaParams];
    }
    elseif ($mediaParams instanceof EntityReferenceFieldItemList) {
      $mediaList = $mediaParams->referencedEntities();
    }
    elseif ($mediaParams instanceof EntityReferenceItem) {
      $mediaList = [
        $this->entityTypeManager->getStorage('media')
          ->load($mediaParams->target_id),
      ];
    }

    // Récupération des traductions.
    $langcode = $this->languageManager->getCurrentLanguage()->getId();

    return array_map(
      function ($media) use ($langcode) {
        return $media->hasTranslation($langcode) ? $media->getTranslation($langcode) : $media;
      },
      $mediaList
    );
  }

  /**
   * Return the build array of the image formatter according to options.
   *
   * @param \Drupal\media\MediaInterface $media
   *   The media.
   * @param array $options
   *   The options.
   *
   * @return array|false
   *   The build array.
   *
   * @throws \Drupal\Core\TypedData\Exception\MissingDataException
   */
  protected function getImageBuildArrayFromMedia(MediaInterface $media, array $options) {
    // Récupération du fichier.
    /** @var \Drupal\file\FileInterface $file */
    $file = $media->get($options[static::OPTION_FIELD_NAME])
      ->referencedEntities()[0];
    if (is_null($file)) {
      return FALSE;
    }

    // Récupération du build de l'élément de base du media.
    $imageFormatterBuild = $media->get($options[static::OPTION_FIELD_NAME])
      ->first()
      ->view();

    // Redéfinition des méta données.
    $this->initImageMeta($imageFormatterBuild, $media, $options);

    // Gestion de l'image style.
    $this->initImageFormatterStyleData($imageFormatterBuild, $file, $options);

    // Gestion des classes.
    $this->initImagesClasses($imageFormatterBuild, $options);

    // Add image tag data.
    $this->initImageTag($imageFormatterBuild);

    // Init attributes.
    $this->initAttributes($imageFormatterBuild, $options);

    // Init srcset.
    $this->initSrcSet($imageFormatterBuild, $media, $options);

    // Init rules.
    $this->initRules($imageFormatterBuild, $media, $options);

    // No size.
    if ($options[static::OPTION_NO_SIZE]) {
      $imageFormatterBuild['#item_attributes'][static::OPTION_NO_SIZE] = 1;
    }

    return $imageFormatterBuild;
  }

  /**
   * Return the image style data.
   *
   * @param array $imageFormatterBuild
   *   The image formatter build array.
   * @param \Drupal\file\FileInterface $file
   *   The file.
   * @param array $options
   *   The options.
   */
  protected function initImageFormatterStyleData(array &$imageFormatterBuild, FileInterface $file, array $options) {
    // Récupération de l'url stylée.
    /** @var \Drupal\image\ImageStyleInterface $style */
    if ($style = $options[static::OPTION_STYLE]) {
      $styledImageUri = $style->buildUri($file->getFileUri());
      $imageFormatterBuild['#image_style'] = $style->id();
      if ($options[static::OPTION_DIMENSION] && file_exists($styledImageUri)) {
        list($image_width, $image_height) = getimagesize($styledImageUri);
        $imageFormatterBuild['#height'] = $image_height;
        $imageFormatterBuild['#width'] = $image_width;
      }
    }
  }

  /**
   * Initialize the image class.
   *
   * @param array $formatterBuild
   *   The formatter build array.
   * @param array $options
   *   The options.
   */
  protected function initImagesClasses(array &$formatterBuild, array $options) {
    if (!empty($options[static::OPTION_CLASS])) {
      $class = is_array($options[static::OPTION_CLASS]) ? $options[static::OPTION_CLASS] : explode(
        ' ',
        $options[static::OPTION_CLASS]
      );

      if (array_key_exists('#attributes', $formatterBuild) && $formatterBuild['#attributes']['class']) {
        $formatterBuild['#attributes']['class'] = array_merge($formatterBuild['#attributes']['class'], $class);
      }
      else {
        $formatterBuild['#attributes']['class'] = $class;
      }
    }
  }

  /**
   * Add base data to the image formatter.
   *
   * @param array $imageFormatterBuild
   *   The image formatter build array.
   */
  protected function initImageTag(array &$imageFormatterBuild) {
    $imageFormatterBuild['#item_attributes']['class'] = array_key_exists(
      '#attributes',
      $imageFormatterBuild
    ) ? $imageFormatterBuild['#attributes']['class'] : [];
    $imageFormatterBuild['#item_attributes']['alt'] = $imageFormatterBuild['#alt'];
  }

  /**
   * Init image attributes.
   *
   * @param array $imageFormatterBuild
   *   The image formatter build array.
   * @param array $options
   *   The options.
   */
  protected function initAttributes(array &$imageFormatterBuild, array $options) {
    $imageFormatterBuild['#item_attributes'] = $options[static::OPTION_ATTRIBUTES] + $imageFormatterBuild['#item_attributes'];
  }

  /**
   * Initialize image meta data.
   *
   * @param array $imageFormatterBuild
   *   The image formatter build array.
   * @param \Drupal\media\MediaInterface $media
   *   The media.
   * @param array $options
   *   The options.
   *
   * @throws \Drupal\Core\TypedData\Exception\MissingDataException
   */
  protected function initImageMeta(array &$imageFormatterBuild, MediaInterface $media, array $options) {
    $imageData = $media->get($options[static::OPTION_FIELD_NAME])
      ->first()
      ->toArray();
    $imageFormatterBuild['#alt'] = empty($imageData['alt']) ? $media->label() : $imageData['alt'];

    // Title.
    if (isset($options[static::OPTION_TITLE]) ||
      isset($imageData['title']) && !empty($imageData['title'])) {
      $imageFormatterBuild['#item_attributes']['title'] = isset($options[static::OPTION_TITLE]) ? $options[static::OPTION_TITLE] : $imageData['title'];
    }
  }

  /**
   * Initialise src set attributes for images.
   *
   * @param array $imageFormatterBuild
   *   The image formatter build array.
   * @param \Drupal\media\Entity\Media $media
   *   The media.
   * @param array $options
   *   The options.
   */
  protected function initSrcSet(array &$imageFormatterBuild, Media $media, array $options) {
    $srcset = $this->getSrcSetFromSourceSettings($media, $options, $options);
    if (!empty($srcset)) {
      array_walk($srcset, function (&$value, $key) {
        $value .= ' ' . $key;
      });
      $imageFormatterBuild['#item_attributes']['srcset'] = implode(', ', $srcset);
    }
  }

  /**
   * Init rules for formater.
   *
   * @param array $imageFormatterBuild
   *   THe build array.
   * @param \Drupal\media\MediaInterface $media
   *   THe media.
   * @param array $options
   *   The options.
   */
  protected function initRules(array &$imageFormatterBuild, MediaInterface $media, array $options) {
    if (isset($options[static::OPTION_RULES])) {
      $settings = [
        static::OPTION_STYLE => $options[static::OPTION_STYLE]->id(),
        static::OPTION_RULES => $options[static::OPTION_RULES],
      ];
      // Get the image file id.
      $fileId = isset($settings['fileId']) ? $settings['fileId'] : $this->getFileIdFromMedia($media, $options);
      $url = $this->imageBuilder->getImageUrlFromSettings($fileId, $settings, [], $options);
      $imageFormatterBuild['#item_attributes'][static::OPTION_OVERRIDE_SRC] = $url;
    }
  }

  /**
   * Return the media url.
   *
   * @param mixed $mediaParams
   *   THe media.
   * @param array $options
   *   The otpions.
   *
   * @return array
   *   The media Urls.
   */
  public function getMediaUrl($mediaParams, array $options = []) {
    // Définitions des options par défaut.
    $options = $this->getDefaultOptions($options);

    // Pour chaque média, on retourne le build array.
    $urls = [];
    foreach ($this->getMediaEntityFromTwigParam($mediaParams) as $media) {
      $urls[$media->id()] = $this->getStyledUrlFromFileId(
        $this->getFileIdFromMedia($media, $options),
        $options[static::OPTION_STYLE],
        $options[static::OPTION_RELATIVE]
      );
    }

    return $urls;
  }

  /**
   * Return the styled url from file Id.
   *
   * @param string|int $fileId
   *   THe file id.
   * @param mixed $style
   *   The style.
   * @param bool $relative
   *   THe url must be relative or not.
   *
   * @return string|null
   *   The file url.
   */
  protected function getStyledUrlFromFileId($fileId, $style = NULL, $relative = TRUE) {
    if ($style) {
      $style = $style instanceof ImageStyleInterface ? $style : $this->conf->getImageStyleStorage()
        ->load($style);
    }
    $url = NULL;
    /** @var \Drupal\file\Entity\File $file */
    if ($file = $this->conf->getFileStorage()->load($fileId)) {
      if ($style) {
        $url = file_create_url($style->buildUri($file->getFileUri()));
      }
      else {
        $url = file_create_url($file->getFileUri());
      }
    }

    if ($relative) {
      $url = file_url_transform_relative($url);
    }

    return $url;
  }

  /**
   * Return the file id from media.
   *
   * @param \Drupal\media\MediaInterface $media
   *   The media.
   * @param array $options
   *   The options.
   *
   * @return string
   *   The file id.
   */
  public function getFileIdFromMedia(MediaInterface $media, array $options) {
    return $media->get($options[static::OPTION_FIELD_NAME])->target_id;
  }

  /**
   * Build picture tag from media.
   *
   * @param mixed $mediaParams
   *   The media params from twig.
   * @param array $options
   *   THe options.
   *
   * @return array
   *   The build array.
   */
  public function pictureFromMedia($mediaParams, array $options) {
    $options = $this->getDefaultOptions($options);
    $picture = [];
    foreach ($this->getMediaEntityFromTwigParam($mediaParams) as $media) {
      $picture[] = $this->getPictureData($media, $options);
    }

    return $picture;
  }

  /**
   * Return picture data.
   *
   * @param \Drupal\media\MediaInterface $media
   *   The media.
   * @param array $options
   *   The options.
   *
   * @return array
   *   The build array.
   */
  protected function getPictureData(MediaInterface $media, array $options) {
    // Init the build array.
    $build = [
      '#theme' => 'picture',
    ];

    // Init main attributes.
    $this->initImagesClasses($build, $options);

    // Init main image tag.
    $build['img'] = $this->imgFromMedia($media, $options);

    // Init sources.
    $build['sources'] = [];
    foreach ($options[static::OPTION_SETTINGS] as $mediaQuery => $settings) {
      $build['sources'][] = $this->getSourceFromSettings($media, $mediaQuery, $settings, $options);
    }

    return $build;
  }

  /**
   * Return the a sourrce from settings.
   *
   * @param \Drupal\media\MediaInterface $media
   *   The media.
   * @param string $mediaQuery
   *   The media query for the source.
   * @param array $settings
   *   The sources settings.
   * @param array $options
   *   The options.
   *
   * @return array
   *   The source build array.
   */
  protected function getSourceFromSettings(MediaInterface $media, $mediaQuery, array $settings, array $options) {
    $srcMedia = $this->getMediaEntityFromTwigParam(isset($settings['media']) ? $settings['media'] : $media);

    return [
      '#theme'       => 'source',
      '#settings'    => $settings,
      '#options'     => $options,
      '#media_query' => $mediaQuery,
      '#src_set'     => $this->getSrcSetFromSourceSettings(reset($srcMedia), $settings, $options),
    ];
  }

  /**
   * Return src set from source settings.
   *
   * @param \Drupal\media\Entity\Media $media
   *   The media.
   * @param array $settings
   *   The src settings.
   * @param array $options
   *   The options.
   *
   * @return array
   *   The list of srcs.
   */
  protected function getSrcSetFromSourceSettings(Media $media, array $settings, array $options) {
    // Get the image file id.
    $fileId = isset($settings['fileId']) ? $settings['fileId'] : $this->getFileIdFromMedia($media, $options);
    $defaultUrl = $this->getStyledUrlFromFileId($fileId, $settings[static::OPTION_STYLE], $options[static::OPTION_RELATIVE]);

    // Build src urls.
    $srcset = [];
    if (isset($settings['srcset'])) {
      foreach ($settings['srcset'] as $type => $data) {
        // Build srcset.
        if ($data === 'default') {
          $srcset[$type] = $defaultUrl;
        }
        elseif ($setUrl = $this->imageBuilder->getImageUrlFromSettings($fileId, $data, $settings, $options)) {
          $srcset[$type] = $setUrl;
        }
      }
    }

    return $srcset;
  }

  /**
   * Return the list of built files from the media.
   *
   * @param \Drupal\media\MediaInterface $media
   *   The media.
   *
   * @return string[]
   *   The files path.
   */
  public function getBuiltFilesFromMedia(MediaInterface $media) {
    $filePaths = [];
    $dirsList = $this->conf->getDirs();

    // Get file ids.
    foreach ($media->toArray() as $fieldName => $value) {
      if (strpos($fieldName, 'field_') === 0 && $media->get($fieldName) instanceof FileFieldItemList) {
        $fileId = $media->get($fieldName)->target_id;
        foreach (['public://', 'private://'] as $dir) {
          try {
            $files = array_filter($this->fileSystem->scanDirectory($dir, '/^' . $fileId . '_/'),
              function ($file) {
                return strpos($file->uri, $this->conf->getDirname()) !== FALSE;
              });
            $filePaths = array_merge($files, $filePaths);
          }
          catch (\Exception $e) {
            // Mute exception...
            $a = 1;
          }
        }
      }
    }

    return $filePaths;
  }

  /**
   * Clear built files.
   *
   * @param \Drupal\media\MediaInterface $media
   *   Media.
   */
  public function clearBuiltFilesFromMedia(MediaInterface $media) {
    foreach ($this->getBuiltFilesFromMedia($media) as $file) {
      $this->fileSystem->delete($file->uri);
    }
  }

}
